﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

using JsDrawControl;
using JsDrawControl.DataSet;
using JsDrawControl.ShortCut;
using JsDrawControl.Util;

namespace JsDraw
{
    //=========================================================================
    /// <summary>
    /// アプリケーションの設定に必要な情報を提供します。
    /// </summary>
    [Serializable]
    public class ConfigInfo : ICloneable
    {
        #region コンストラクタ、デストラクタ

        //=====================================================================
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ConfigInfo()
        {
            // メンバ変数初期化
            this.ImageQuarity = InterpolationMode.HighQualityBicubic;
            this.SmoothingMode = SmoothingMode.AntiAlias;
            this.PanelOffset = new Point(5, 5);
            this.HistoryCount = 50;
            this.ScreenBufferSize = 2;
            this.AdjusterMoveStyle = AdjusterMoveStyle.Simple;
            this.ShowVScrollBar = ShowType.Auto;
            this.ShowHScrollBar = ShowType.Auto;
            this.ShowRuler = true;
            this.CornerText = "㎜";
            this.MinZoom = 0.1F;
            this.MaxZoom = 5.0F;
            this.DefaultPaperSize = PaperKind.A4;
            this.ShortCut = new ShortCutManager();
            this.MultiSelect = true;
            this.UsePentab = false;
            this.EraserSize = 3;

            this.FileHistory = new List<string>();
            this.MainSplitterDistance = 500;
            this.ItemSplitterDistance = 250;
            this.WindowRectangle = new Rectangle(100, 100,800,600);
            this.WindowState = FormWindowState.Normal;

            this.ClipBoardDataFormat = ClipBoardManager.FormatType.ImageAndExtendMeta;
            this.ClipBoardImageFormatSize = 3.0F;

            this.PrinterName = string.Empty;
            this.PrintOrderRightBottom = true;
            this.PrintCutLine = false;
            this.PrintGlueLine = false;
        }

        #endregion

        #region 表示プロパティ

        //=====================================================================
        /// <summary>
        /// イメージクオリティプロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("イメージ描画のクオリティを設定します。"),
         System.ComponentModel.DefaultValueAttribute(typeof(InterpolationMode), "HighQualityBicubic")]
        public InterpolationMode ImageQuarity { get; set; }

        //=====================================================================
        /// <summary>
        /// スムージングモードプロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("描画のスムージングモードを設定します。"),
         System.ComponentModel.DefaultValueAttribute(typeof(SmoothingMode), "AntiAlias")]
        public SmoothingMode SmoothingMode { get; set; }

        //=====================================================================
        /// <summary>
        /// パネル描画オフセットプロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("パネルを描画するオフセット値を設定します。")]
        public Point PanelOffset { get; set; }

        //=====================================================================
        /// <summary>
        /// 垂直スクロールバー表示プロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("縦スクロールバーの表示を設定します。"),
         System.ComponentModel.DefaultValueAttribute(typeof(ShowType), "Auto")]
        public ShowType ShowVScrollBar { get; set; }

        //=====================================================================
        /// <summary>
        /// 水平スクロールバー表示プロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("横スクロールバーの表示を設定します。"),
         System.ComponentModel.DefaultValueAttribute(typeof(ShowType), "Auto")]
        public ShowType ShowHScrollBar { get; set; }

        //=====================================================================
        /// <summary>
        /// ルーラーの表示プロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("ルーラーの表示を設定します。"),
         System.ComponentModel.DefaultValueAttribute(true)]
        public bool ShowRuler { get; set; }

        //=====================================================================
        /// <summary>
        /// コーナーのテキストプロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("コーナーのテキストを設定します。")]
        public string CornerText { get; set; }

        //=====================================================================
        /// <summary>
        /// デフォルト用紙サイズプロパティ
        /// </summary>
        [System.ComponentModel.Category("デザイン"),
         System.ComponentModel.Description("デフォルトの用紙サイズを設定します。")]
        public PaperKind DefaultPaperSize { get; set; }

        //=====================================================================
        /// <summary>
        /// 操作履歴件数プロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("操作履歴蓄積件数を取得または設定します。このプロパティに設定した回数だけ操作のやり直しが行えます。０以下の値を設定すると操作履歴の蓄積を行いません。"),
         System.ComponentModel.DefaultValueAttribute(50)]
        public int HistoryCount { get; set; }

        //=====================================================================
        /// <summary>
        /// スクリーンバッファサイズプロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("スクリーンバッファサイズを取得または設定します。\nコントロールと同じサイズにしたい場合には 1 を指定します。以降２乗倍ずつ増加します。（2を指定すると4面分、3を指定すると9面分... ）"),
         System.ComponentModel.DefaultValueAttribute(2)]
        public int ScreenBufferSize { get; set; }

        //=====================================================================
        /// <summary>
        /// アジャスター移動スタイルプロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("アジャスターの移動スタイルを取得または設定します。"),
         System.ComponentModel.DefaultValueAttribute(typeof(AdjusterMoveStyle), "Simple")]
        public AdjusterMoveStyle AdjusterMoveStyle { get; set; }

        //=====================================================================
        /// <summary>
        /// ズーム最小値プロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("マウスホイールを回して縮小した場合にストップをかける最小ズームレベルを取得または設定します。"),
         System.ComponentModel.DefaultValueAttribute(0.01F)]
        public float MinZoom { get; set; }

        //=====================================================================
        /// <summary>
        /// ズーム最大値プロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("マウスホイールを回して拡大した場合にストップをかける最大ズームレベルを取得または設定します。"),
         System.ComponentModel.DefaultValueAttribute(10.0F)]
        public float MaxZoom { get; set; }

        //=====================================================================
        /// <summary>
        /// ショートカットプロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("ショートカットを取得、または設定します。")]
        public ShortCutManager ShortCut { get; set; }

        //=====================================================================
        /// <summary>
        /// マルチ選択プロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("アイテムを複数選択できるかどうかを設定します。"),
         System.ComponentModel.DefaultValueAttribute(true)]
        public bool MultiSelect { get; set; }

        //=====================================================================
        /// <summary>
        /// ペンタブレット使用プロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("ペンタブレットを使用するかどうか設定します。"),
         System.ComponentModel.DefaultValueAttribute(false)]
        public bool UsePentab { get; set; }


        //=====================================================================
        /// <summary>
        /// 消しゴムサイズプロパティ
        /// </summary>
        [System.ComponentModel.Category("コントロール"),
         System.ComponentModel.Description("消しゴムのサイズを 1～5 の範囲で設定します。"),
         System.ComponentModel.DefaultValueAttribute(3)]
        public int EraserSize { get; set; }

        //=====================================================================
        /// <summary>
        /// データフォーマットタイププロパティ
        /// </summary>
        [System.ComponentModel.Category("クリップボード"),
         System.ComponentModel.Description("クリップボードに設定するフォーマットを取得または設定します。"),
         System.ComponentModel.DefaultValueAttribute(typeof(ClipBoardManager.FormatType), "ImageAndExtendMeta")]
        public ClipBoardManager.FormatType ClipBoardDataFormat{get;set;}

        //=====================================================================
        /// <summary>
        /// イメージフォーマットサイズプロパティ
        /// </summary>
        [System.ComponentModel.Category("クリップボード"),
         System.ComponentModel.Description("イメージフォーマットでセットされるイメージサイズを取得または設定します。"),
         System.ComponentModel.DefaultValueAttribute(3.0F)]
        public float ClipBoardImageFormatSize { get; set; }

        #endregion

        #region 非表示プロパティ

        //=====================================================================
        /// <summary>
        /// ファイルヒストリプロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public List<string> FileHistory { get; set; }

        //=====================================================================
        /// <summary>
        /// メインスプリッタの距離プロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public int MainSplitterDistance { get; set; }

        //=====================================================================
        /// <summary>
        /// アイテムスプリッタの距離プロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public int ItemSplitterDistance { get; set; }

        //=====================================================================
        /// <summary>
        /// ウインドウレクトプロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public Rectangle WindowRectangle { get; set; }

        //=====================================================================
        /// <summary>
        /// ウィンドウステートプロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public System.Windows.Forms.FormWindowState WindowState { get; set; }

        //=====================================================================
        /// <summary>
        /// プリンタ名プロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public string PrinterName { get; set; }

        //=====================================================================
        /// <summary>
        /// 印刷方向プロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public bool PrintOrderRightBottom { get; set; }

        //=====================================================================
        /// <summary>
        /// 切り取り線印刷プロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public bool PrintCutLine { get; set; }

        //=====================================================================
        /// <summary>
        /// のりしろ印刷プロパティ
        /// </summary>
        [System.ComponentModel.Browsable(false)]
        public bool PrintGlueLine { get; set; }

        #endregion

        #region ICloneable

        //=====================================================================
        /// <summary>
        /// クローン作成
        /// </summary>
        /// <returns></returns>
        public virtual object Clone()
        {
            ConfigInfo value = new ConfigInfo();

            // メンバ変数初期化
            value.ImageQuarity = this.ImageQuarity;
            value.SmoothingMode = this.SmoothingMode;
            value.PanelOffset = this.PanelOffset;
            value.ShowVScrollBar = this.ShowVScrollBar;
            value.ShowHScrollBar = this.ShowHScrollBar;
            value.ShowRuler = this.ShowRuler;
            value.CornerText = this.CornerText;
            value.FileHistory.Clear();
            foreach(string path in this.FileHistory)
                value.FileHistory.Add(path);
            value.MainSplitterDistance = this.MainSplitterDistance;
            value.ItemSplitterDistance = this.ItemSplitterDistance;
            value.WindowRectangle = this.WindowRectangle;
            value.WindowState = this.WindowState;
            value.ScreenBufferSize = this.ScreenBufferSize;
            value.AdjusterMoveStyle = this.AdjusterMoveStyle;
            value.MaxZoom = this.MaxZoom;
            value.MinZoom = this.MinZoom;
            value.DefaultPaperSize = this.DefaultPaperSize;
            value.ShortCut = this.ShortCut.Clone() as ShortCutManager;
            value.MultiSelect = this.MultiSelect;
            value.UsePentab = this.UsePentab;
            value.EraserSize = this.EraserSize;

            return value;
        }

        #endregion
    }
}
