﻿using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing.Drawing2D;
using System.Collections.Generic;
using System.Threading;

using JsDrawControl;
using JsDrawControl.Item;
using JsDrawControl.Panel;
using JsDrawControl.History;
using JsDrawControl.ShortCut;
using System.Drawing.Printing;
using JsDrawControl.DataSet;

namespace JsDraw
{
    public partial class MainForm : Form
    {
        #region 定数定義

        private const float MM_INCHE = 3.937007874015748F;

        #endregion

        #region メンバ変数

        /// <summary>イベント処理を行うかどうかを制御します。</summary>
        /// <remarks>true:イベント処理を行う、false:イベント処理を行わない</remarks>
        private bool _eventFlag;

        /// <summary>ファイル履歴</summary>
        private ToolStripMenuItem[] _fileHistory = null;

        /// <summary>プロパティ変更開始フラグ</summary>
        /// <remarks>プロパティ変更が開始されている場合は true を設定</remarks>
        private bool _chgPropertyStartFlag;

        /// <summary>アクティブパネル</summary>
        private DesignPanel _activePanel;

        // リスト選択変更イベント時間
        private DateTime _listSelectedChangeEventTime;

        // リスト選択タイマー
        private System.Threading.Timer _listSelectedTimer;

        // リスト選択タイマーフラグ
        private bool _listSelectedTimerFlag;

        #endregion

        #region コンストラクタ

        //=====================================================================
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            // メンバ変数初期化
            this._eventFlag = false;
            _listSelectedChangeEventTime = DateTime.Now;
            _listSelectedTimer = new System.Threading.Timer(ListSelectedTimerProc);
            _listSelectedTimerFlag = false;

            // ヒストリメニュー作成
            this._fileHistory = new ToolStripMenuItem[5];
            this._fileHistory[0] = this.menuFileHistory1;
            this._fileHistory[1] = this.menuFileHistory2;
            this._fileHistory[2] = this.menuFileHistory3;
            this._fileHistory[3] = this.menuFileHistory4;
            this._fileHistory[4] = this.menuFileHistory5;

            this._chgPropertyStartFlag = false;
            _activePanel = null;

        }

        #endregion

        #region フォームロードイベント

        //=====================================================================
        /// <summary>
        /// フォームロードイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // 画面項目クリア
            this.statusMousePoint.Text = string.Empty;
            this.statusSelectBounds.Text = string.Empty;
            this.statuslPaperSize.Text = string.Empty;

            // タイトル設定
            this.Text = Program.PROGRAM_TITLE;

            // イベント処理を行うようにフラグを設定
            this._eventFlag = true;

            // 新しいドキュメントを作成
            this.NewDoc();

            // ファイル履歴メニュー設定
            this.SetFileHistMenu();

            // ボタン設定
            this.toolItemDefault.Checked = true;
        }

        #endregion

        #region フォームクローズイベント

        //=====================================================================
        /// <summary>
        /// フォームクローズ確認イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 内容が変更されている場合は保存する。
            IEnumerable<DesignPanel> panels =
                from DesignPanel n in this.jscSplitPanel.Panels
                where n.Drawview.IsChanged
                select n;

            foreach (DesignPanel panel in panels)
            {
                // 保存確認
                DialogResult result = MessageBox.Show(
                    this,
                    MessageDef.GetMessage(MessageDef.I001001,
                        string.IsNullOrEmpty(panel.FileName) ? "新規" : Path.GetFileName(panel.FileName)),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.YesNoCancel,
                    MessageBoxIcon.Question);

                switch (result)
                {
                    case DialogResult.Yes:
                        // 保存に失敗した場合、クローズキャンセル
                        if (!this.Save(panel))
                            e.Cancel = true;
                        break;

                    case DialogResult.Cancel:
                        // クローズキャンセル
                        e.Cancel = true;
                        break;
                }

                // キャンセルされた場合
                if (e.Cancel)
                    break;
            }
        }

        //=====================================================================
        /// <summary>
        /// フォームクローズイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // ========== 画面状態の保存 ==========
            // メインスプリッタの距離
            Program.Config.MainSplitterDistance = this.splitMain.SplitterDistance;

            // アイテムスプリッタの距離
            Program.Config.ItemSplitterDistance = this.splitItem.SplitterDistance;

            // 画面位置
            Program.Config.WindowRectangle =
                new Rectangle(
                    this.Location.X, this.Location.Y,
                    this.Size.Width, this.Size.Height);

            // 画面状態
            Program.Config.WindowState = this.WindowState;
        }

        #endregion

        #region 「新規作成」ボタンクリックイベント
        
        //=====================================================================
        /// <summary>
        /// 「新規作成」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonNew_Click(object sender, EventArgs e)
        {
            // 新規作成
            this.NewDoc(); 
        }

        #endregion

        #region 「開く」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「開く」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOpen_Click(object sender, EventArgs e)
        {
            // 読み込み処理
            this.Read();
        }

        #endregion

        #region 「保存」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「保存」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 上書き保存
            this.Save(_activePanel);
        }

        #endregion

        #region 「ページ設定」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ページ設定」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPageSetting_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            if ((_activePanel.Drawview.Panel is PaperPanel) == false)
            {
                MessageBox.Show(this, "このパネルオブジェクトはページ設定できません。",
                    Program.PROGRAM_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            // ページ設定
            PageSettingForm frm = new PageSettingForm();
            PaperPanel panel = _activePanel.Drawview.Panel as PaperPanel;

            // データを画面に設定
            frm.PaperKind = panel.PaperKind;
            frm.PaperOrientation = panel.PaperOrientation;
            frm.PaperSize = panel.PaperSize;
            frm.ColCount = panel.ColCount;
            frm.RowCount = panel.RowCount;
            frm.PaperMargin = panel.Margin;
            frm.MarginSize = panel.MarginSize;

            // 画面を表示する。
            if (frm.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    _activePanel.Drawview.BeginInit();

                    // フォームデータを設定
                    panel.PaperKind = frm.PaperKind;
                    panel.PaperOrientation = frm.PaperOrientation;
                    if( frm.PaperKind == JsDrawControl.PaperKind.Custom)
                        panel.PaperSize = frm.PaperSize;
                    panel.ColCount = frm.ColCount;
                    panel.RowCount = frm.RowCount;
                    panel.Margin = frm.PaperMargin;
                    panel.MarginSize = frm.MarginSize;
                }
                finally
                {
                    _activePanel.Drawview.EndInit();
                }
            }
        }

        #endregion

        #region 「印刷」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「印刷」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPrint_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            if ((_activePanel.Drawview.Panel is PaperPanel) == false)
            {
                MessageBox.Show(this, "このパネルオブジェクトは印刷できません。",
                    Program.PROGRAM_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            using (PrintForm frm = new PrintForm())
            {
                using (PrintDocument doc = this.GetPrintDocument())
                {
                    // ダイアログ設定
                    frm.PrintDoc = doc;
                    frm.PrintOrderRightBottom = Program.Config.PrintOrderRightBottom;
                    frm.PrintCutLine = Program.Config.PrintCutLine;
                    frm.PrintGlueLine = Program.Config.PrintGlueLine;

                    if (frm.ShowDialog(this) == DialogResult.OK)
                    {
                        // ダイアログから設定情報を取得
                        Program.Config.PrinterName = frm.PrintDoc.PrinterSettings.PrinterName;
                        Program.Config.PrintOrderRightBottom = frm.PrintOrderRightBottom;
                        Program.Config.PrintCutLine = frm.PrintCutLine;
                        Program.Config.PrintGlueLine = frm.PrintGlueLine;

                        if (frm.ShowPreview)
                        {
                            // プレビュー表示
                            using (PrintPreviewDialog prv = new PrintPreviewDialog())
                            {
                                // だいあろぐ設定
                                prv.Document = doc;
                                prv.StartPosition = FormStartPosition.Manual;
                                prv.Location = this.Location;
                                prv.Size = this.Size;
                                prv.ShowIcon = false;

                                // ダイアログ表示
                                prv.ShowDialog(this);
                            }
                        }
                        else
                        {
                            // 印刷
                            doc.Print();                            
                        }
                    }
                }
            }
        }

        #endregion

        #region 「カーソル」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「カーソル」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolItemDefault_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // デフォルトモード設定
            _activePanel.Drawview.ControlMode = ControlMode.Default;
            this.toolItemDefault.Checked = true;
        }

        #endregion

        #region 「ビュー」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ビュー」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonView_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // デフォルトモード設定
            _activePanel.Drawview.ControlMode = ControlMode.View;
            this.toolItemView.Checked = true;
        }

        #endregion

        #region 「移動」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「移動」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHand_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // ハンドモード設定
            _activePanel.Drawview.ControlMode = ControlMode.Hnad;
            this.toolItemHand.Checked = true;
        }

        #endregion

        #region 「ズーム」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ズーム」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonZoom_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // ハンドモード設定
            _activePanel.Drawview.ControlMode = ControlMode.Zoom;
            this.toolItemZoom.Checked = true;
        }

        #endregion

        #region 「アイテム選択」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「アイテム選択」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemSelect_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // ハンドモード設定
            _activePanel.Drawview.ControlMode = ControlMode.Select;
            this.toolItemSelect.Checked = true;
        }

        #endregion

        #region 「消しゴム」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「消しゴム」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolErase_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 消しゴムモード設定
            _activePanel.Drawview.ControlMode = ControlMode.Erase;
            this.toolErase.Checked = true;
        }

        #endregion

        #region 「縮小」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「縮小」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonZoomOut_Click(object sender, EventArgs e)
        {
            // ズームコンボボックス設定
            int index = this.toolZoom.SelectedIndex - 1;

            if (0 <= index && index < this.toolZoom.Items.Count)
                this.toolZoom.SelectedIndex = index;
        }

        #endregion

        #region 「拡大」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「拡大」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonZoomIn_Click(object sender, EventArgs e)
        {
            // ズームコンボボックス設定
            int index = this.toolZoom.SelectedIndex + 1;

            if (0 <= index && index < this.toolZoom.Items.Count)
                this.toolZoom.SelectedIndex = index;
        }

        #endregion

        #region 「元に戻す」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「元に戻す」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonUndo_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 元に戻す
            if (_activePanel.Drawview.CanUndo())
                _activePanel.Drawview.Undo();
        }

        #endregion

        #region 「やり直し」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「やり直し」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRedo_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // やり直し
            if (_activePanel.Drawview.CanRedo())
                _activePanel.Drawview.ReDo();
        }

        #endregion

        #region 「削除」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「削除」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonDelete_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 選択されているアイテムを削除
            _activePanel.Drawview.RemoveSelectedItem();
        }

        #endregion

        #region 「前面に移動」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「前面に移動」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOrderUp_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 選択されているアイテムを移動
            _activePanel.Drawview.MoveItemOrder(ItemOrderDirection.Up);
        }

        #endregion

        #region 「背面に移動」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「背面に移動」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOrderDown_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 選択されているアイテムを移動
            _activePanel.Drawview.MoveItemOrder(ItemOrderDirection.Down);
        }

        #endregion

        #region 「切り取り」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「切り取り」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCut_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 選択されているアイテムを切り取り
            _activePanel.Drawview.CutItem();
        }
        
        #endregion

        #region 「コピー」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「コピー」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCopy_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 選択されているアイテムをコピー
            _activePanel.Drawview.CopyItem();
        }
        
        #endregion

        #region 「貼り付け」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「貼り付け」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPaste_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 選択されているアイテムを貼り付け
            _activePanel.Drawview.PasteItem();
        }

        #endregion

        #region 「名前を付けて保存」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「名前を付けて保存」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSaveAs_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // 名前をつけて保存
            this.SaveAs(_activePanel);
        }

        #endregion

        #region 「ファイル履歴」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ファイル履歴」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonFileHistory_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;

            // ファイル名取得
            string fileName = (string)item.Tag;

            // ファイル存在チェック
            if (!File.Exists(fileName))
            {
                // エラーメッセージ表示
                MessageBox.Show(
                    this,
                    MessageDef.GetMessage(
                        MessageDef.E001003,
                        fileName),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }

            // ファイル読み込み
            this.Read(fileName);
        }

        #endregion

        #region 「終了」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「終了」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion

        #region 「ヘルプ」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ヘルプ」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHelpFile_Click(object sender, EventArgs e)
        {
            MessageBox.Show(this, "ヘルプはまだ作成していません。",
                Program.PROGRAM_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #endregion

        #region 「バージョン」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「バージョン」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonVersion_Click(object sender, EventArgs e)
        {
            VersionForm ver = new VersionForm();
            ver.ShowDialog(this);
        }

        #endregion

        #region 「設定」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// ツール設定ボタンクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolSetting_Click(object sender, EventArgs e)
        {
            ToolSettingForm setting = new ToolSettingForm();
            setting.Config = Program.Config;

            // 設定画面表示
            if (setting.ShowDialog(this) == DialogResult.OK)
            {
                Program.Config = setting.Config;
                this.SettingForm(Program.Config, false);
            }
        }

        #endregion

        #region [操作履歴クリア]ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// [操作履歴クリア]ボタンクリックイベント処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClearHistory_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            if (MessageBox.Show(this, "操作履歴をクリアしてもよろしいですか？", 
                Program.PROGRAM_TITLE, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                return;

            // コマンド履歴クリア
            _activePanel.Drawview.CommandHistoryClear();

            // ヒストリリスト更新
            this.SetHistoryList();
        }
        #endregion

        #region アイテムツールボタンイベント処理

        //=====================================================================
        /// <summary>
        /// ラインアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemLine_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // 直線アイテム作成
            LineItem item = new LineItem();
            item.Name = "直線";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemLine.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 曲線アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemPolyLine_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // 曲線アイテム作成
            PolyLineItem item = new PolyLineItem();
            item.Name = "曲線";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemPolyLine.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// フリーアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemFreeLine_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // フリーアイテム作成
            FreeLineItem item = new FreeLineItem();
            item.Name = "ペン";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemFreeLine.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 円アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemCircle_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // 円アイテム作成
            EllipseItem item = new EllipseItem();
            item.Name = "円";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemCircle.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 三角アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemTriangle_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // 三角形アイテム作成
            TrianguleItem item = new TrianguleItem();
            item.Name = "三角形";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemTriangle.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 四角アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemRectangle_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // 四角アイテム作成
            RectItem item = new RectItem();
            item.Name = "四角形";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemRectangle.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// ラウンドアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemRound_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // ラウンドアイテム作成
            RoundItem item = new RoundItem();
            item.Name = "ラウンド";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemRound.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 多角形アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemPolygon_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // 多角形アイテム作成
            PolygonItem item = new PolygonItem();
            item.Name = "多角形";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemPolygon.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// グループアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemGroup_Click(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            this.UnCheckItemButton();

            // イメージアイテム作成
            GroupItem item = new GroupItem();
            item.Name = "グループ";

            // アイテムをデザインモードで追加
            _activePanel.Drawview.AddDrawItem(item, true);
            this.toolItemGroup.Checked = true;
        }
        
        #endregion

        #region [J.S Drawコントロール]　マウス移動イベント

        //=====================================================================
        /// <summary>
        /// マウス移動イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_MouseMove(object sender, MouseEventArgs e)
        {
            if (_activePanel == null)
                return;

            DrawPanel panel = (DrawPanel)_activePanel.Drawview.Panel;

            // マウス位置をスケール座標に変換する。
            PointF scrPoint = _activePanel.Drawview.GetScalePoint(e.Location);

            // マージンサイズ
            SizeF mz = new SizeF(0, 0);
            if (panel.GridBase == GridBase.Margin)
                mz = new SizeF(panel.MarginSize.Left, panel.MarginSize.Top);

            this.statusMousePoint.Text =
                string.Format(" X:{0:0.0:}, Y:{1:0.0} (mm)",
                scrPoint.X - mz.Width,
                scrPoint.Y - mz.Height);

            if (_activePanel.Drawview.SpinAngle != 0.0F)
            {
                // ※回転角度プロパティが設定されている場合はアイテム回転中

                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[2];

                // テキスト設定
                this.statusSelectBounds.Text =
                    string.Format(" {0:0.00}°", _activePanel.Drawview.SpinAngle);
            }
            else if (_activePanel.Drawview.MouseFrameRect != RectangleF.Empty)
            {
                // ※マウス選択範囲プロパティが設定されている場合はマウスで範囲選択中

                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[0];

                // テキスト設定
                this.statusSelectBounds.Text =
                    string.Format(" X:{0:0.0}, Y:{1:0.0} (mm) - W:{2:0.0}, H:{3:0.0} (mm)",
                    _activePanel.Drawview.MouseFrameRect.X - mz.Width,
                    _activePanel.Drawview.MouseFrameRect.Y - mz.Height,
                    _activePanel.Drawview.MouseFrameRect.Width,
                    _activePanel.Drawview.MouseFrameRect.Height);
            }
            else if (_activePanel.Drawview.SelectedItemBounds != RectangleF.Empty)
            {
                // ※選択アイテム範囲が設定されている場合はアイテムが選択されている。

                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[1];

                // テキスト設定
                this.statusSelectBounds.Text =
                    string.Format(" X:{0:0.0}, Y:{1:0.0} (mm) - W:{2:0.0}, H:{3:0.0} (mm)",
                    _activePanel.Drawview.SelectedItemBounds.X - mz.Width,
                    _activePanel.Drawview.SelectedItemBounds.Y - mz.Height,
                    _activePanel.Drawview.SelectedItemBounds.Width,
                    _activePanel.Drawview.SelectedItemBounds.Height);
            }
            else
            {
                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[0];
                // テキスト設定
                this.statusSelectBounds.Text = string.Empty;
            }
        }

        #endregion

        #region [J.S Drawコントロール]　コントロールモード変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　コントロールモード変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_ControlModeChanged(
            object sender, JsDrawControl.ControlModeChangedEventArgs e)
        {
            // コントロールモードボタン設定
            this.SetControlModeButton(e.ControlMode);
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテム追加イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテム追加イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemInsert(object sender, JsDrawControl.DrawItemInsertEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();

            // タイトル設定
            this.SetTitle();
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテム追加キャンセルイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテム追加キャンセルイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_CancelDrawItemInsert(object sender, JsDrawControl.CancelDrawItemInsertEventArgs e)
        {
            // アイテムツールボタンのチェックを外す
            this.UnCheckItemButton();
        }

        #endregion

        #region [J.S Drawコントロール]　ズーム率変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　ズーム率変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_ZoomChanged(
            object sender, JsDrawControl.ZoomChangedEventArgs e)
        {
            try
            {
                // コンボボックスのイベントを発生させなくするため、イベントフラグを false に設定する
                this._eventFlag = false;

                // ズーム設定
                this.SetZoom(e.Zoom);
            }
            finally
            {
                // イベントフラグを戻す
                this._eventFlag = true;
            }
        }

        #endregion

        #region [J.S Drawコントロール]　選択描画アイテム変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　選択描画アイテム変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemSelectedChanged(
            object sender, JsDrawControl.DrawItemSelectedChangedEventArgs e)
        {
            JsDrawView view = sender as JsDrawView;

            // ユーザーによって行われた操作の場合
            if (e.Action == DrawItemAction.ByUser)
            {
                // プロパティグリッド設定
                this.SetPropertyGrid();

                // アイテムリスト更新
                this.UpdateItemListSelect();
            }
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテム変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテム変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemChanged(
            object sender, JsDrawControl.DrawItemChangedEventArgs e)
        {
            JsDrawView view = sender as JsDrawView;

            // ユーザーによって行われた操作の場合
            if (e.Action == DrawItemAction.ByUser)
            {
                // アイテムリスト更新
                this.UpdateItemList();

                // プロパティグリッド更新
                this.UpdatePropertyGrid();

                // 履歴リスト設定
                this.SetHistoryList();
            }
            else
            {
                // ユーザー操作でない場合はプロパティ変更フラグを true に設定する
                // 複数アイテムを選択した状態でプロパティグリッドの値を変更すると描画に時間がかかるため、
                // "chgPropertyStartFlag" フラグを使って一括で描画が行えるようにします。
                if (this._chgPropertyStartFlag == false)
                {
                    this._chgPropertyStartFlag = true;
                    view.BeginInit();
                }
            }

            // タイトル設定
            this.SetTitle();
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテムリムーブイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテムリムーブイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemRemove(
            object sender, JsDrawControl.DrawItemRemoveEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // プロパティグリッド更新
            this.UpdatePropertyGrid();

            // ヒストリリスト更新
            this.SetHistoryList();

            // タイトル設定
            this.SetTitle();
        }

        #endregion

        #region [J.S Drawコントロール]　描画パネル変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画パネル変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_PanelItemChanged(
            object sender, JsDrawControl.PanelChangedEventArgs e)
        {
            // ステータスバー設定
            this.SetPaperSizeStatus();

            // プロパティグリッド更新
            this.UpdatePropertyGrid();

            // ヒストリリスト更新
            this.SetHistoryList();

            // タイトル設定
            this.SetTitle();
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテムオーダー変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテムオーダー変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemChangeOrder(
            object sender, JsDrawControl.DrawItemChangeOrderEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();

            // タイトル設定
            this.SetTitle();
        }

        #endregion

        #region [J.S Drawコントロール]　やり直しイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　やり直しイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_ReDoing(
            object sender, JsDrawControl.ReDoingEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　元に戻すイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　元に戻すイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_UnDoing(
            object sender, JsDrawControl.UnDoingEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　マウスクリックイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　マウスクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_MouseClick(object sender, MouseEventArgs e)
        {

        }

        #endregion

        #region [J.S Drawコントロール]　マウスダブルクリックイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　マウスダブルクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_MouseDoubleClick(object sender, MouseEventArgs e)
        {

        }

        #endregion

        #region [アイテムリスト]　選択インデックス変更イベント

        //=====================================================================
        /// <summary>
        /// [アイテムリスト]　選択インデックス変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listItem_SelectedIndexChanged(object sender, EventArgs e)
        {
            // イベントを処理しない場合は終了
            if ((_activePanel == null) || (_eventFlag == false))
                return;

            // リスト選択変更イベント時間
            _listSelectedChangeEventTime = DateTime.Now;

            // リスト選択タイマー設定
            if( _listSelectedTimerFlag == false)
                _listSelectedTimer.Change(100, Timeout.Infinite);
        }

        //=====================================================================
        /// <summary>
        /// リスト選択タイマーイベントを処理します。
        /// </summary>
        /// <param name="state"></param>
        private void ListSelectedTimerProc(object state)
        {
            try
            {
                // マルチ選択時に処理時間がかかるので前回選択された時から 0.2 秒以上
                // 経過していないと処理しません。
                TimeSpan span = DateTime.Now - _listSelectedChangeEventTime;

                if (200 < span.TotalMilliseconds)
                {
                    // 一覧選択処理
                    if (this.listItem.InvokeRequired)
                    {
                        Action proc = new Action(SelectedItemFromList);
                        this.listItem.Invoke(proc);
                    }
                    _listSelectedTimerFlag = false;
                }
                else
                {
                    // 次のタイマーを設定
                    _listSelectedTimer.Change(100, Timeout.Infinite);
                }
            }
            catch
            {
                // エラー発生時にはフラグを戻す
                _listSelectedTimerFlag = false;
            }
        }

        //=====================================================================
        /// <summary>
        /// 一覧の内容でアイテムを選択します。
        /// </summary>
        private void SelectedItemFromList()
        {
            if (_activePanel == null)
                return;

            try
            {
                _activePanel.Drawview.BeginInit();

                // アイテム選択クリア
                _activePanel.Drawview.SelectedItems.Clear();

                foreach (ListViewItem listItem in this.listItem.Items)
                {
                    // アイテムリストで選択中のアイテムを DrawView でも選択する。
                    if (listItem.Selected)
                        _activePanel.Drawview.SelectedItems.Add(listItem.Tag as IDrawItem);
                }

                // プロパティグリッド設定
                if (0 < _activePanel.Drawview.SelectedItems.Count)
                {
                    this.propertyItem.SelectedObjects = _activePanel.Drawview.SelectedItems.ToArray();
                }
                else
                {
                    this.propertyItem.SelectedObject = _activePanel.Drawview.Panel;
                }
            }
            finally
            {
                _activePanel.Drawview.EndInit();
            }
        }

        #endregion

        #region [プロパティグリッド]　プロパティ値変更イベント

        //=====================================================================
        /// <summary>
        /// [プロパティグリッド]　プロパティ値変更イベント処理を行います。
        /// </summary>
        /// <param name="s"></param>
        /// <param name="e"></param>
        private void propertyItem_PropertyValueChanged(
            object s, PropertyValueChangedEventArgs e)
        {
            if (_activePanel == null)
                return;

            // 複数アイテムを選択した状態でプロパティグリッドの値を変更すると描画に時間がかかるため、
            // "chgPropertyStartFlag" フラグを使って一括で描画が行えるようにします。
            if (this._chgPropertyStartFlag)
            {
                this._chgPropertyStartFlag = false;
                _activePanel.Drawview.EndInit();

                // アイテムリスト更新
                this.SetItemList();

                // プロパティグリッド更新
                this.UpdatePropertyGrid();

                // ヒストリリスト更新
                this.SetHistoryList();
            }
        }

        #endregion

        #region [ズームコンボボックス]　選択インデックス変更イベント

        //=====================================================================
        /// <summary>
        /// ズームコンボボックス選択インデックス変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolZoom_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_activePanel == null)
                return;

            // イベントを処理しない場合は終了
            if (this._eventFlag == false)
                return;

            if (this.toolZoom.SelectedItem != null)
            {
                ListItem item = (ListItem)this.toolZoom.SelectedItem;

                // 拡大率の設定
                _activePanel.Drawview.Zoom = item.FloatValue;
            }
        }

        #endregion

        #region [SplitPanelコントロール]　イベント

        //=====================================================================
        /// <summary>
        /// 選択パネル変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void jscSplitPanel_PanelSelectedChanged(
            object sender, JsControlLibrary.EventArgs.PanelSelectedChangedEventArgs e)
        {
            if (_activePanel != null)
            {
                // イベント削除
                _activePanel.Drawview.MouseMove -=
                    new MouseEventHandler(drawView_MouseMove);
                _activePanel.Drawview.ControlModeChanged -=
                    new JsDrawView.ControlModeChanged_EventHandler(drawView_ControlModeChanged);
                _activePanel.Drawview.DrawItemInsert -=
                    new JsDrawView.DrawItemInsert_EventHandler(drawView_DrawItemInsert);
                _activePanel.Drawview.CancelDrawItemInsert -=
                    new JsDrawView.CancelDrawItemInsert_EventHandler(drawView_CancelDrawItemInsert);
                _activePanel.Drawview.ZoomChanged -=
                    new JsDrawView.ZoomChanged_EventHandler(drawView_ZoomChanged);
                _activePanel.Drawview.DrawItemSelectedChanged -=
                    new JsDrawView.DrawItemSelectedChanged_EventHandler(drawView_DrawItemSelectedChanged);
                _activePanel.Drawview.DrawItemChanged -=
                    new JsDrawView.DrawItemChanged_EventHandler(drawView_DrawItemChanged);
                _activePanel.Drawview.DrawItemRemove -=
                    new JsDrawView.DrawItemRemove_EventHandler(drawView_DrawItemRemove);
                _activePanel.Drawview.PanelItemChanged -=
                    new JsDrawView.PanelItemChanged_EventHandler(drawView_PanelItemChanged);
                _activePanel.Drawview.DrawItemChangeOrder -=
                    new JsDrawView.DrawItemChangeOrder_EventHandler(drawView_DrawItemChangeOrder);
                _activePanel.Drawview.ReDoing -=
                    new JsDrawView.ReDoing_EventHandler(drawView_ReDoing);
                _activePanel.Drawview.UnDoing -=
                    new JsDrawView.UnDoing_EventHandler(drawView_UnDoing);
                _activePanel.Drawview.MouseClick -=
                    new MouseEventHandler(drawView_MouseClick);
                _activePanel.Drawview.MouseDoubleClick -=
                    new MouseEventHandler(drawView_MouseDoubleClick);
            }

            if (e.Index < 0)
            {
                _activePanel = null;
            }
            else
            {
                // アクティブパネル設定
                _activePanel = e.Panel as DesignPanel;

                // イベント追加
                _activePanel.Drawview.MouseMove += 
                    new MouseEventHandler(drawView_MouseMove);
                _activePanel.Drawview.ControlModeChanged += 
                    new JsDrawView.ControlModeChanged_EventHandler(drawView_ControlModeChanged);
                _activePanel.Drawview.DrawItemInsert += 
                    new JsDrawView.DrawItemInsert_EventHandler(drawView_DrawItemInsert);
                _activePanel.Drawview.CancelDrawItemInsert += 
                    new JsDrawView.CancelDrawItemInsert_EventHandler(drawView_CancelDrawItemInsert);
                _activePanel.Drawview.ZoomChanged += 
                    new JsDrawView.ZoomChanged_EventHandler(drawView_ZoomChanged);
                _activePanel.Drawview.DrawItemSelectedChanged += 
                    new JsDrawView.DrawItemSelectedChanged_EventHandler(drawView_DrawItemSelectedChanged);
                _activePanel.Drawview.DrawItemChanged += 
                    new JsDrawView.DrawItemChanged_EventHandler(drawView_DrawItemChanged);
                _activePanel.Drawview.DrawItemRemove += 
                    new JsDrawView.DrawItemRemove_EventHandler(drawView_DrawItemRemove);
                _activePanel.Drawview.PanelItemChanged += 
                    new JsDrawView.PanelItemChanged_EventHandler(drawView_PanelItemChanged);
                _activePanel.Drawview.DrawItemChangeOrder += 
                    new JsDrawView.DrawItemChangeOrder_EventHandler(drawView_DrawItemChangeOrder);
                _activePanel.Drawview.ReDoing += 
                    new JsDrawView.ReDoing_EventHandler(drawView_ReDoing);
                _activePanel.Drawview.UnDoing += 
                    new JsDrawView.UnDoing_EventHandler(drawView_UnDoing);
                _activePanel.Drawview.MouseClick +=
                    new MouseEventHandler(drawView_MouseClick);
                _activePanel.Drawview.MouseDoubleClick +=
                    new MouseEventHandler(drawView_MouseDoubleClick);
            }

            try
            {
                _eventFlag = false;

                // アイテムリスト設定
                this.SetItemList();

                // プロパティグリッド設定
                this.SetPropertyGrid();

                // 操作履歴リスト設定
                this.SetHistoryList();

                // 操作履歴詳細設定
                this.SetHistoryDetail();

                // ズーム設定
                this.SetZoom(_activePanel.Drawview.Zoom);

                // モード設定
                _activePanel.Drawview.ControlMode = ControlMode.Default;
                this.SetControlModeButton(ControlMode.Default);

                // タイトル設定
                this.SetTitle();
            }
            finally
            {
                _eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// パネルクローズイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void jscSplitPanel_ClosePanel(
            object sender, JsControlLibrary.EventArgs.ClosePanelEventArgs e)
        {
            DesignPanel panel = e.Panel as DesignPanel;

            if (panel.Drawview.IsChanged)
            {
                // 変更されている場合は保存の確認を行う
                DialogResult result = MessageBox.Show(
                    this,
                    MessageDef.GetMessage(MessageDef.I001001,
                        string.IsNullOrEmpty(panel.FileName) ? "新規" : Path.GetFileName(panel.FileName)),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.YesNoCancel,
                    MessageBoxIcon.Question);

                switch (result)
                {
                    case DialogResult.Yes:
                        // 保存に失敗した場合、クローズキャンセル
                        if (!this.Save(panel))
                            e.Cancel = true;
                        break;

                    case DialogResult.Cancel:
                        // クローズキャンセル
                        e.Cancel = true;
                        break;
                }
            }

            // パネル無しの場合
            if ((e.Cancel == false) && this.jscSplitPanel.Panels.Count == 1)
            {
                _activePanel = null;
                this.SetControlModeButton(ControlMode.Default);

                // 画面設定
                this.SetItemList();
                this.SetPropertyGrid();
                this.SetHistoryList();
                this.SetHistoryDetail();
                this.SetTitle();
            }
        }

        #endregion

        #region [操作タブ]　選択インデックス変更イベント

        //=====================================================================
        /// <summary>
        /// 操作タブインデックス変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabOperation_SelectedIndexChanged(object sender, EventArgs e)
        {
            // アイテムリスト設定
            this.SetItemList();

            // 操作履歴リスト設定
            this.SetHistoryList();
        }

        #endregion

        #region プライベートメソッド

        //=====================================================================
        /// <summary>
        /// ツールバーのアイテムボタンのチェックをすべて外します。
        /// </summary>
        private void UnCheckItemButton()
        {
            // ツールバーのボタンを戻す
            foreach (ToolStripItem item in this.toolItem.Items)
            {
                if (item is ToolStripButton)
                {
                    var btn = item as ToolStripButton;

                    if (btn.Checked)
                        btn.Checked = false;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// ツールバーのズームボタンの設定を行います。
        /// </summary>
        /// <param name="zoom"></param>
        private void SetZoom(float zoom)
        {
            // アイテム検索
            int index = -1;
            for (int i = 0; i < this.toolZoom.Items.Count; i++)
            {
                index = i;
                ListItem item = (ListItem)this.toolZoom.Items[i];
                if (zoom <= item.FloatValue)
                    break;
            }

            // コンボボックス設定
            if (this.toolZoom.SelectedIndex != index)
                this.toolZoom.SelectedIndex = index;

            // ========== 拡大、縮小ボタン設定 ==========
            // 縮小ボタン設定
            if (0 < this.toolZoom.SelectedIndex)
            {
                this.toolZoomOut.Enabled = true;
            }
            else
            {
                this.toolZoomOut.Enabled = false;
            }

            // 拡大ボタン設定
            if (this.toolZoom.SelectedIndex < this.toolZoom.Items.Count - 1)
            {
                this.toolZoomIn.Enabled = true;
            }
            else
            {
                this.toolZoomIn.Enabled = false;
            }
        }

        //=====================================================================
        /// <summary>
        /// 用紙サイズステータスバーの設定を行います。
        /// </summary>
        private void SetPaperSizeStatus()
        {
            if (_activePanel == null)
            {
                this.statuslPaperSize.Text = "";
            }
            else
            {
                this.statuslPaperSize.Text =
                    string.Format(" 用紙サイズ：横 {0}mm × 縦 {1}mm",
                    _activePanel.Drawview.Panel.PanelSize.Width,
                    _activePanel.Drawview.Panel.PanelSize.Height);
            }
        }

        //=====================================================================
        /// <summary>
        /// タイトル文字列の設定を行います。
        /// </summary>
        private void SetTitle()
        {
            string fileName = string.Empty;

            if (_activePanel == null)
            {
                fileName = string.Empty;
            }
            else
            {
                if (_activePanel.FileName == string.Empty)
                {
                    fileName = "新規" + (_activePanel.IsChanged ? " *" : "");
                }
                else
                {
                    fileName = _activePanel.FileName + (_activePanel.IsChanged ? " *" : "");
                }
            }

            this.Text = string.Format("{0} [{1}]", Program.PROGRAM_TITLE, fileName);
        }

        //=====================================================================
        /// <summary>
        /// ファイル履歴メニューの設定を行います。
        /// </summary>
        private void SetFileHistMenu()
        {
            for (int i = 1; i <= 5; i++)
            {
                int index = i - 1;

                if (i <= Program.Config.FileHistory.Count)
                {
                    _fileHistory[index].Visible = true;
                    _fileHistory[index].Text =
                        string.Format("&{0} {1}", i, Program.Config.FileHistory[index]);
                    _fileHistory[index].Tag = Program.Config.FileHistory[index];
                }
                else
                {
                    _fileHistory[index].Visible = false;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// コントロールモードボタン設定処理
        /// </summary>
        /// <param name="mode"></param>
        private void SetControlModeButton(ControlMode mode)
        {
            // ボタンチェック状態
            bool chkDef = false;
            bool chkHand = false;
            bool chkZoom = false;
            bool chkSelect = false;
            bool chkView = false;
            bool chkErase = false;
            bool chkTool = false;

            // コントロールモードボタン設定
            switch (mode)
            {
                case ControlMode.Default:   // デフォルトモード
                case ControlMode.ItemEdit:  // アイテム編集モード
                    chkDef = true;
                    break;

                case ControlMode.Hnad:      // ハンドモード
                    chkHand = true;
                    break;

                case ControlMode.Zoom:      // ズームモード
                    chkZoom = true;
                    break;

                case ControlMode.Select:    // 選択モード
                    chkSelect = true;
                    break;

                case ControlMode.View:      // ビューモード
                    chkView = true;
                    break;

                case ControlMode.Erase:     // 消しゴムモード
                    chkErase = true;
                    break;

                case ControlMode.ItemAdd:   // アイテム追加モード
                    chkTool = true;
                    break;
            }

            // ========== ボタン状態設定 ==========
            // デフォルト
            this.toolItemDefault.Checked = chkDef;

            // 移動
            this.toolItemHand.Checked = chkHand;

            // ズーム
            this.toolItemZoom.Checked = chkZoom;

            // 選択範囲
            this.toolItemSelect.Checked = chkSelect;

            // ビュー
            this.toolItemView.Checked = chkView;

            // 消しゴム
            this.toolErase.Checked = chkErase;

            // ツールバーのボタンを戻す
            if (chkTool == false)
            {
                this.toolItemLine.Checked = false;
                this.toolItemPolyLine.Checked = false;
                this.toolItemFreeLine.Checked = false;
                this.toolItemCircle.Checked = false;
                this.toolItemTriangle.Checked = false;
                this.toolItemRectangle.Checked = false;
                this.toolItemRound.Checked = false;
                this.toolItemPolygon.Checked = false;
                this.toolItemGroup.Checked = false;
            }
        }

        #endregion

        #region フォーム設定処理

        //=====================================================================
        /// <summary>
        /// フォームを設定します。
        /// </summary>
        /// <param name="config">設定</param>
        /// <param name="setFormStatus">フォームステータスを設定</param>
        public void SettingForm(ConfigInfo config, bool setFormStatus)
        {
            try
            {
                // イベント処理を行うようにフラグを設定
                this._eventFlag = false;

                // ========== ズームコンボボックス設定処理 ==========
                // 設定前の値を取得する
                float befValue = 1.0F;
                if (this.toolZoom.SelectedItem != null)
                    befValue = (this.toolZoom.SelectedItem as ListItem).FloatValue;

                this.toolZoom.Items.Clear();

                // コンボボックス設定
                int minZoom = (int)(config.MinZoom * 100F);
                int maxZoom = (int)(config.MaxZoom * 100F);
                for(int z= minZoom ; z <= maxZoom ; z += 5)
                    this.toolZoom.Items.Add(new ListItem(string.Format("{0}%", z), (float)z / 100F));

                // 値の範囲確認
                if (befValue < config.MinZoom)befValue = config.MinZoom;
                if (befValue > config.MaxZoom)befValue = config.MaxZoom;

                // コンボボックス選択
                SetZoom(befValue);

                foreach (DesignPanel panel in this.jscSplitPanel.Panels)
                {
                    // DrawView 設定
                    this.SetConfig(config, panel.Drawview);

                    // ステータスバー設定
                    this.SetPaperSizeStatus();

                    if (setFormStatus)
                    {
                        // ========== メイン画面設定 ==========
                        // ウインドウ内に収まるかチェックする
                        bool sizeChkFlag = false;

                        foreach (Screen sc in Screen.AllScreens)
                        {
                            if (sc.WorkingArea.Contains(config.WindowRectangle))
                            {
                                sizeChkFlag = true;
                                break;
                            }
                        }

                        // ウインドウに収まらない場合は初期値
                        if (sizeChkFlag == false)
                            config.WindowRectangle = new Rectangle(100, 100, 800, 600);

                        // ウィンドウロケーション
                        this.Location = new Point(config.WindowRectangle.X, config.WindowRectangle.Y);

                        // ウィンドウサイズ
                        this.Size = new Size(config.WindowRectangle.Width, config.WindowRectangle.Height);

                        // ウインドウステート
                        if (config.WindowState == FormWindowState.Minimized)
                        {
                            this.WindowState = FormWindowState.Normal;
                        }
                        else
                        {
                            this.WindowState = config.WindowState;
                        }

                        // メインスプリッタの距離
                        this.splitMain.SplitterDistance = config.MainSplitterDistance;

                        // アイテムスプリッタの設定
                        this.splitItem.SplitterDistance = config.ItemSplitterDistance;
                    }

                    // ========== ショートカット設定 ==========
                    foreach (ShortCutKey shortCut in panel.Drawview.ShortCut.CommandKeyList)
                    {
                        switch (shortCut.ShortCutCommand)
                        {
                            case ShortCutCommand.Cut:
                                this.SetMenuShortCutKey(this.menuCut, shortCut);
                                break;

                            case ShortCutCommand.Copy:
                                this.SetMenuShortCutKey(this.menuCopy, shortCut);
                                break;

                            case ShortCutCommand.Paste:
                                this.SetMenuShortCutKey(this.menuPaste, shortCut);
                                break;

                            case ShortCutCommand.UnDo:
                                this.SetMenuShortCutKey(this.menuUndo, shortCut);
                                break;

                            case ShortCutCommand.ReDo:
                                this.SetMenuShortCutKey(this.menuRedo, shortCut);
                                break;
                        }
                    }
                }
            }
            finally
            {
                // イベント処理を行うようにフラグを設定
                this._eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// 設定情報をビューに反映します。
        /// </summary>
        /// <param name="item">メニューアイテム</param>
        /// <param name="shortcutKey">ショートカットキー</param>
        private void SetConfig(ConfigInfo config, JsDrawView view)
        {
            // DrawView 設定
            view.ImageQuarity = config.ImageQuarity;
            view.SmoothingMode = config.SmoothingMode;
            view.PanelOffset = config.PanelOffset;
            view.CommnadHistoryCount = config.HistoryCount;
            view.ScreenBufferSize = config.ScreenBufferSize;
            view.AdjusterMoveStyle = config.AdjusterMoveStyle;
            view.ShowVScrollBar = config.ShowVScrollBar;
            view.ShowHScrollBar = config.ShowHScrollBar;
            view.ShowRuler = config.ShowRuler;
            view.CornerText = config.CornerText;
            view.MinZoom = config.MinZoom;
            view.MaxZoom = config.MaxZoom;
            view.ClipBoardDataFormat = config.ClipBoardDataFormat;
            view.ClipBoardImageFormatSize = config.ClipBoardImageFormatSize;
            view.UsePenTab = config.UsePentab;
            view.EraserSize = config.EraserSize;

            // ショートカット設定
            if (0 == config.ShortCut.CommandKeyList.Count)
            {
                config.ShortCut = view.ShortCut.Clone() as ShortCutManager;
            }
            else
            {
                view.ShortCut.CommandKeyList = config.ShortCut.CommandKeyList;
            }

            view.ShortCut.Enabled = config.ShortCut.Enabled;
            view.MultiSelect = config.MultiSelect;
        }

        //=====================================================================
        /// <summary>
        /// メニューのショートカット設定します。
        /// </summary>
        /// <param name="item">メニューアイテム</param>
        /// <param name="shortcutKey">ショートカットキー</param>
        private void SetMenuShortCutKey(ToolStripMenuItem item, ShortCutKey shortcutKey)
        {
            Keys menuKey = Keys.None;

            if (shortcutKey != null && shortcutKey.Enabled)
                menuKey = shortcutKey.GetShortCutKeys();

            item.ShortcutKeys = menuKey;
        }

        #endregion

        #region 新規ドキュメント作成

        //=====================================================================
        /// <summary>
        /// ドキュメントの新規作成を行います。
        /// </summary>
        private void NewDoc()
        {
            // 新規パネル作成
            DesignPanel designPanel = new DesignPanel();
            designPanel.Title = "新規";
            designPanel.FileName = string.Empty;
            designPanel.FormatType = FormatType.jsd;

            // 新規ドキュメント
            PaperPanel panel = new PaperPanel();

            // 用紙サイズはＡ４
            panel.PaperKind = Program.Config.DefaultPaperSize;

            // 新しいドキュメントを作成
            designPanel.Drawview.NewDocument(panel);

            // DrawView 設定
            this.SetConfig(Program.Config, designPanel.Drawview);

            // パネルの追加
            this.jscSplitPanel.Panels.Add(designPanel);
            this.jscSplitPanel.SelectedPanel = designPanel;

            // 更新フラグ設定
            designPanel.Drawview.IsChanged = false;

            // タイトル設定
            this.SetTitle();

            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();

            // プロパティグリッド設定
            this.propertyItem.SelectedObject = designPanel.Drawview.Panel;
        }

        #endregion

        #region 保存処理

        //=====================================================================
        /// <summary>
        /// 保存を行います。
        /// </summary>
        /// <param name="panel">パネル</param>
        /// <returns>キャンセルされた場合 false</returns>
        private bool Save(DesignPanel panel)
        {
            bool ret = true;

            if (File.Exists(panel.FileName))
            {
                try
                {
                    // マウスカーソル設定
                    this.Cursor = Cursors.WaitCursor;

                    // ファイル保存
                    using (FileStream st = new FileStream(panel.FileName, FileMode.Create, FileAccess.Write))
                    {
                        panel.Drawview.Save(st, panel.FormatType);
                    }

                    // 更新フラグ設定
                    panel.Drawview.IsChanged = false;
                    panel.IsChanged = false;

                    // ファイル履歴設定
                    this.SetFileHistory(panel.FileName);

                    // ファイル履歴メニュー設定処理
                    this.SetFileHistMenu();

                    // タイトル設定
                    this.SetTitle();
                }
                catch (Exception ex)
                {
                    // エラーログ出力
                    LogWriter.WriteExceptionLog(ex);

                    // エラーメッセージ表示
                    MessageBox.Show(
                        this,
                        MessageDef.GetMessage(
                            MessageDef.E001001,
                            ex.ToString()),
                        Program.PROGRAM_TITLE,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
                finally
                {
                    // マウスカーソルを戻す
                    this.Cursor = Cursors.Default;
                }
            }
            else
            {
                // ファイル名が指定されていない場合は名前を付けて保存
                ret = SaveAs(panel);
            }

            return ret;
        }

        #endregion

        #region 名前を付けて保存

        //=====================================================================
        /// <summary>
        /// 名前を付けて保存を行います。
        /// </summary>
        /// <param name="panel">パネル</param>
        /// <returns>キャンセルされた場合 false</returns>
        private bool SaveAs(DesignPanel panel)
        {
            bool ret = false;

            using (SaveFileDialog dlog = new SaveFileDialog())
            {
                // ダイアログ設定
                dlog.Filter = "J.S Draw 形式ファイル|*.jsd|ＸＭＬ形式ファイル|*.xml";
                dlog.FileName = string.IsNullOrEmpty(panel.FileName) ? string.Empty : Path.GetFileNameWithoutExtension(panel.FileName);
                dlog.OverwritePrompt = true;
                dlog.CheckPathExists = true;

                // フィルターインデックス設定
                if (!string.IsNullOrEmpty(panel.FileName))
                {
                    switch (Path.GetExtension(panel.FileName))
                    {
                        case ".jsd":
                            // Jsl 形式
                            dlog.FilterIndex = 1;
                            break;

                        case ".xml":
                            // Xml 形式
                            dlog.FilterIndex = 2;
                            break;

                        default:
                            // Jsl 形式
                            dlog.FilterIndex = 1;
                            break;
                    }
                }

                if (dlog.ShowDialog(this) == DialogResult.OK)
                {
                    FormatType fmt = FormatType.jsd;

                    // フォーマットタイプ
                    switch (dlog.FilterIndex)
                    {
                        case 1:
                            // Jsl 形式
                            fmt = FormatType.jsd;
                            break;

                        case 2:
                            // Xml 形式
                            fmt = FormatType.xml;
                            break;
                    }

                    try
                    {
                        // マウスカーソル設定
                        this.Cursor = Cursors.WaitCursor;

                        // ファイル保存
                        using (FileStream st = new FileStream(dlog.FileName, FileMode.Create, FileAccess.Write))
                        {
                            panel.Drawview.Save(st, fmt);
                        }

                        // ロードファイル名設定
                        panel.FileName = dlog.FileName;

                        // ロードファイルフォーマット設定
                        panel.FormatType = fmt;

                        // 更新フラグ設定
                        panel.Drawview.IsChanged = false;
                        panel.IsChanged = false;

                        // タブ設定
                        panel.Title = Path.GetFileName(dlog.FileName);

                        // タイトル設定
                        this.SetTitle();

                        // ファイル履歴設定
                        this.SetFileHistory(dlog.FileName);

                        // ファイル履歴メニュー設定処理
                        this.SetFileHistMenu();

                        // タイトル設定
                        this.SetTitle();

                        ret = true;
                    }
                    catch (Exception ex)
                    {
                        // エラーログ出力
                        LogWriter.WriteExceptionLog(ex);

                        // エラーメッセージ表示
                        MessageBox.Show(
                            this,
                            MessageDef.GetMessage(
                                MessageDef.E001001,
                                ex.ToString()),
                            Program.PROGRAM_TITLE,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
                    }
                    finally
                    {
                        // マウスカーソルを戻す
                        this.Cursor = Cursors.Default;
                    }
                }
            }

            return ret;
        }

        #endregion

        #region 読み込み

        //=====================================================================
        /// <summary>
        /// ファイルダイアログを表示してドキュメントの読み込みを行います。
        /// </summary>
        private void Read()
        {
            using (OpenFileDialog dlog = new OpenFileDialog())
            {
                // ダイアログ設定
                dlog.Filter = "J.S Print2 形式ファイル|*.jsd|ＸＭＬ形式ファイル|*.xml";
                dlog.FileName = _activePanel != null ? _activePanel.FileName : string.Empty;
                dlog.CheckFileExists = true;
                dlog.CheckPathExists = true;

                if (dlog.ShowDialog(this) == DialogResult.OK)
                {
                    // ファイル読み込み
                    this.Read(dlog.FileName);
                }
            }
        }


        //=====================================================================
        /// <summary>
        /// 指定されたファイルの読み込みを行います。
        /// </summary>
        /// <param name="fileName">読み込みファイル名</param>
        private void Read(string fileName)
        {
            try
            {
                // パネルの作成
                DesignPanel panel = new DesignPanel();

                // マウスカーソル設定
                this.Cursor = Cursors.WaitCursor;

                FormatType fmt = FormatType.jsd;

                // フォーマットタイプ
                switch (Path.GetExtension(fileName).ToLower())
                {
                    case ".jsd":
                        // Jsl 形式
                        fmt = FormatType.jsd;
                        break;

                    case ".xml":
                        // Xml 形式
                        fmt = FormatType.xml;
                        break;
                }

                // ファイル読み込み
                using (FileStream st = new FileStream(fileName, FileMode.Open, FileAccess.Read))
                {
                    panel.Drawview.Read(st, fmt);
                }

                // ========== パネルオブジェクトチェック ==========
                // 用紙パネルでない場合。
                if ((panel.Drawview.Panel is PaperPanel) != true)
                {
                    // 保存確認
                    DialogResult result = MessageBox.Show(
                        this,
                        MessageDef.GetMessage(MessageDef.I001002),
                        Program.PROGRAM_TITLE,
                        MessageBoxButtons.YesNoCancel,
                        MessageBoxIcon.Question);

                    if (result == System.Windows.Forms.DialogResult.Yes)
                    {
                        // パネルオブジェクト変換
                        DrawPanel oldPanel = panel.Drawview.Panel as DrawPanel;
                        PaperPanel newPanel = new PaperPanel();

                        newPanel.PaperKind = JsDrawControl.PaperKind.Custom;
                        newPanel.PaperSize = oldPanel.PanelSize;
                        newPanel.Margin = oldPanel.Margin;
                        newPanel.MarginSize = oldPanel.MarginSize;
                        newPanel.DrawMargin = oldPanel.DrawMargin;
                        newPanel.MarginLineDataSet = (LineDataSet)oldPanel.MarginLineDataSet.Clone();
                        newPanel.DrawGlid = oldPanel.DrawGlid;
                        newPanel.GridOver = oldPanel.GridOver;
                        newPanel.GridSize = oldPanel.GridSize;
                        newPanel.GridBase = oldPanel.GridBase;
                        newPanel.FirstGridLineInterval = oldPanel.FirstGridLineInterval;
                        newPanel.SecondGridLineInterval = oldPanel.SecondGridLineInterval;
                        newPanel.FirstGridLineDataSet = (LineDataSet)oldPanel.FirstGridLineDataSet.Clone();
                        newPanel.SecondGridLineDataSet = (LineDataSet)oldPanel.SecondGridLineDataSet;

                        // パネル設定
                        panel.Drawview.Panel = newPanel;
                        oldPanel.Dispose();
                    }
                }

                // ロードファイル名設定
                panel.FileName = fileName;

                // ロードーファイルフォーマット設定
                panel.FormatType = fmt;

                // 更新フラグ設定
                panel.Drawview.IsChanged = false;
                panel.IsChanged = false;

                // DrawView 設定
                this.SetConfig(Program.Config, panel.Drawview);

                // 新規パネルしかない場合削除する。
                if (this.jscSplitPanel.Panels.Count == 1)
                {
                    DesignPanel p = this.jscSplitPanel.Panels[0] as DesignPanel;
                    if (p.Drawview.Items.Count == 0)
                        this.jscSplitPanel.Panels.Clear();
                }

                // パネルの追加
                panel.Title = Path.GetFileName(fileName);
                this.jscSplitPanel.Panels.Add(panel);
                this.jscSplitPanel.SelectedPanel = panel;

                // タイトル設定
                this.SetTitle();

                // アイテムリスト更新
                this.SetItemList();

                // ヒストリリスト更新
                this.SetHistoryList();

                // プロパティグリッド設定
                this.propertyItem.SelectedObject = panel.Drawview.Panel;
            }
            catch (Exception ex)
            {
                // エラーログ出力
                LogWriter.WriteExceptionLog(ex);

                // エラーメッセージ表示
                MessageBox.Show(
                    this,
                    MessageDef.GetMessage(
                        MessageDef.E001002,
                        ex.ToString()),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            finally
            {
                // マウスカーソルを戻す
                this.Cursor = Cursors.Default;
            }
        }

        #endregion

        #region ファイル履歴設定

        //=====================================================================
        /// <summary>
        /// ファイル履歴設の理を行います。
        /// </summary>
        /// <param name="fileName">ファイル名</param>
        private void SetFileHistory(string fileName)
        {
            int index = -1;

            // すでに登録されていないかチェックする
            for (int i = 0; i < Program.Config.FileHistory.Count; i++)
            {
                if (fileName == Program.Config.FileHistory[i])
                {
                    index = i;
                    break;
                }
            }

            // すでに登録されている場合、削除する
            if (0 <= index)
                Program.Config.FileHistory.RemoveAt(index);

            // リストの先頭に登録する
            Program.Config.FileHistory.Insert(0, fileName);

            // ５件を超える場合削除
            int cnt = Program.Config.FileHistory.Count - 5;
            for (int i = 0; i < cnt; i++)
                Program.Config.FileHistory.RemoveAt(Program.Config.FileHistory.Count - 1);

            // 設定ファイル保存
            Program.WriteSettings();
        }

        #endregion

        #region アイテムリスト設定

        //=====================================================================
        /// <summary>
        /// アイテムリストの設定を行います。
        /// </summary>
        private void SetItemList()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if ((_activePanel == null) || (this.tabOperation.SelectedIndex != 0))
            {
                this.listItem.Items.Clear();
                return;
            }

            // アイテム数が０の場合はリストクリア
            if (_activePanel.Drawview.Items.Count == 0)
            {
                this.listItem.Items.Clear();
                return;
            }

            try
            {
                this.listItem.BeginUpdate();

                // アイテムリストの選択変更イベントを処理しないようにする
                this._eventFlag = false;

                // リストクリア
                this.listItem.Items.Clear();

                // アイテムリスト追加
                this.AddItemList(_activePanel.Drawview.Items);
            }
            finally
            {
                this.listItem.EndUpdate();

                // イベントフラグを戻す
                this._eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// アイテムリスト追加を行います。
        /// </summary>
        /// <param name="collection">追加するアイテムのコレクション</param>
        private void AddItemList(DrawItemCollection collection)
        {
            IEnumerable<IDrawItem> items = 
                from IDrawItem n in collection
                orderby collection.IndexOf(n) descending
                select n;

            ListViewItem topItem = null;

            foreach (IDrawItem item in items)
            {
                // リストビューアイテム
                ListViewItem listItem = new ListViewItem();
                listItem.Tag = item;

                // アイテムイメージ
                listItem.ImageIndex = this.GetItemImageIndex(item);

                // アイテム名
                listItem.SubItems.Add(this.GetItemIndexTree(item) + item.Name);

                // アイテム位置
                listItem.SubItems.Add(
                    string.Format("{0,4:0},{1,4:0}",
                    item.Location.X, item.Location.Y));

                // サブアイテム追加
                listItem.SubItems.Add(string.Empty);

                // 選択されている場合
                if (item.IsSelected)
                {
                    listItem.Selected = true;
                    if (topItem == null)
                        topItem = listItem;
                }

                // アイテム追加
                this.listItem.Items.Add(listItem);

                // グループアイテムの場合、再帰呼び出し
                if (item is GroupItem)
                {
                    GroupItem g = item as GroupItem;

                    if (0 < g.Items.Count)
                        this.AddItemList(g.Items);
                }
            }

            this.listItem.TopItem = topItem;
        }

        //=====================================================================
        /// <summary>
        /// アイテムリストの更新を行います。
        /// </summary>
        public void UpdateItemList()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if (this.tabOperation.SelectedIndex != 0)
                return;

            try
            {
                this.listItem.BeginUpdate();

                // アイテムリストの選択変更イベントを処理しないようにする
                this._eventFlag = false;
                ListViewItem topItem = null;

                foreach (ListViewItem listItem in this.listItem.Items)
                {
                    IDrawItem item = (IDrawItem)listItem.Tag;

                    // アイテムイメージ
                    listItem.ImageIndex = this.GetItemImageIndex(item);

                    if (listItem.Selected && topItem == null)
                        topItem = listItem;

                    // アイテム名
                    listItem.SubItems[1].Text = this.GetItemIndexTree(item) + item.Name;

                    // アイテム位置
                    listItem.SubItems[2].Text =
                        string.Format("{0,4:0},{1,4:0}",
                        item.Location.X, item.Location.Y);
                }

                this.listItem.TopItem = topItem;
            }
            finally
            {
                this.listItem.EndUpdate();

                // イベントフラグを戻す
                this._eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// アイテムリスト選択状態の更新を行います。
        /// </summary>
        public void UpdateItemListSelect()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if (this.tabOperation.SelectedIndex != 0)
                return;

            try
            {
                this.listItem.BeginUpdate();

                // アイテムリストの選択変更イベントを処理しないようにする
                this._eventFlag = false;
                ListViewItem topItem = null;

                foreach (ListViewItem listItem in this.listItem.Items)
                {
                    // 選択されている場合
                    if (_activePanel.Drawview.SelectedItems.Contains(listItem.Tag as IDrawItem))
                    {
                        listItem.Selected = true;
                        if( topItem == null)
                            topItem = listItem;
                    }
                    else
                    {
                        listItem.Selected = false;
                    }
                }

                this.listItem.TopItem = topItem;
            }
            finally
            {
                this.listItem.EndUpdate();

                // イベントフラグを戻す
                this._eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// アイテムイメージインデックスを取得します。
        /// </summary>
        /// <param name="item">インデックスを取得するアイテム</param>
        /// <returns>アイテムインデックス</returns>
        private int GetItemImageIndex(IDrawItem item)
        {
            int imageIndex = 0;

            // イメージがロックされている場合は開始インデックスを変更する。
            if (item.InheritanceDataSet.GetLock())
                imageIndex = 9;

            if (item is LineItem)
            {
                // 直線アイテム
                imageIndex += 0;
            }
            else if (item is PolyLineItem)
            {
                // 曲線アイテム
                imageIndex += 1;
            }
            else if (item is FreeLineItem)
            {
                // フリー
                imageIndex += 2;
            }
            else if (item is EllipseItem)
            {
                // 円
                imageIndex += 3;
            }
            else if (item is TrianguleItem)
            {
                // 三角形
                imageIndex += 4;
            }
            else if (item is RectItem)
            {
                // 四角アイテム
                imageIndex += 5;
            }
            else if (item is RoundItem)
            {
                // ラウンドアイテム
                imageIndex += 6;
            }
            else if (item is PolygonItem)
            {
                // 多角形アイテム
                imageIndex += 7;
            }
            else if (item is GroupItem)
            {
                // グループアイテム
                imageIndex += 8;
            }

            return imageIndex;
        }
        
        //=====================================================================
        /// <summary>
        /// アイテムインデックスのツリーを取得します。
        /// </summary>
        /// <param name="item">ツリーを取得するアイテム</param>
        /// <returns></returns>
        public string GetItemIndexTree(IDrawItem item)
        {
            // アイテムのインデックスを取得する。
            DrawItemIndex index = _activePanel.Drawview.Items.ItemIndexOf(item);

            // 深度が０の場合は終了
            if (index.Depth == 0)
                return string.Empty;

            StringBuilder sb = new StringBuilder();

            for (int i = 1; i <= index.Depth; i++)
            {
                if (i == index.Depth)
                {
                    if (index[i] == 0)
                    {
                        sb.Append("└ ");
                    }
                    else
                    {
                        sb.Append("├ ");
                    }
                }
                else
                {
                    if (index[i] == 0)
                    {
                        sb.Append("　 ");
                    }
                    else
                    {
                        sb.Append("│ ");
                    }
                }
            }

            return sb.ToString();
        }

        //=====================================================================
        /// <summary>
        /// プロパティグリッド更新
        /// </summary>
        private void UpdatePropertyGrid()
        {
            // プロパティグリッドリフレッシュ
            if (this.tabOperation.SelectedIndex == 0)
                this.propertyItem.Refresh();
        }
        
        #endregion

        #region 操作履歴設定

        //=====================================================================
        /// <summary>
        /// 操作履歴リストの設定を行います。
        /// </summary>
        private void SetHistoryList()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if (this.tabOperation.SelectedIndex != 1)
                return;

            if ((_activePanel == null) || (_activePanel.Drawview.CommandHistory.HistoryDataList.Count == 0))
            {
                this.listHistory.Items.Clear();
                this.textDetail.Text = string.Empty;
                this.toolHistoryCount.Text = "履歴件数：0";
                return;
            }

            // 操作履歴データ取得
            CommandHistoryData[] histData =
                _activePanel.Drawview.CommandHistory.HistoryDataList.ToArray();
            Array.Reverse(histData);

            // 操作履歴数が０の場合はリストクリア
            if (histData == null || histData.Length == 0)
            {
                this.listHistory.Items.Clear();
                this.textDetail.Text = string.Empty;
                this.toolHistoryCount.Text = "履歴件数：0";
                return;
            }

            try
            {
                this.listHistory.BeginUpdate();

                int selectedIndex = -1;
                if (this.listHistory.SelectedItems != null && 0 < this.listHistory.SelectedItems.Count)
                    selectedIndex = this.listHistory.Items.IndexOf(this.listHistory.SelectedItems[0]);

                // リストクリア
                this.listHistory.Items.Clear();

                // ========== 操作履歴リスト追加 ==========
                int index = histData.Length-1;
                int histIndex = _activePanel.Drawview.CommandHistory.CurrentIndex;
                foreach( CommandHistoryData data in histData)
                {
                    // リストビューアイテム
                    ListViewItem listItem = new ListViewItem(string.Empty, index == histIndex ? 0 : -1);
                    listItem.Tag = data;

                    listItem.SubItems.Add(data.CommandName);
                    listItem.SubItems.Add(data.ComanndDataList.Count.ToString());
                    this.listHistory.Items.Add(listItem);
                    index--;
                }

                // 行選択
                if (0 <= selectedIndex && index < this.listHistory.Items.Count)
                    this.listHistory.Items[selectedIndex].Selected = true;

                this.toolHistoryCount.Text = string.Format("履歴件数：{0}", histData.Length);
            }
            finally
            {
                this.listHistory.EndUpdate();
            }

            // 履歴詳細設定
            this.SetHistoryDetail();
        }

        //=====================================================================
        /// <summary>
        /// 履歴詳細を設定します。
        /// </summary>
        private void SetHistoryDetail()
        {
            StringBuilder sb = new StringBuilder();

            if (0 < this.listHistory.SelectedItems.Count)
            {
                // 選択アイテム取得
                ListViewItem item = this.listHistory.SelectedItems[0];

                // コマンドデータ
                CommandHistoryData histData = item.Tag as CommandHistoryData;

                sb.AppendLine(string.Format("CommandName :{0}", histData.CommandName));
                sb.AppendLine(string.Format("ControlMode :{0}", histData.ControlMode.ToString()));
                sb.AppendLine(string.Format("SelectedItem:{0}",
                    histData.SelectedItemList == null ? 0 : histData.SelectedItemList.Length));
                sb.AppendLine(string.Format("CommandCount:{0}",
                    histData.ComanndDataList == null ? 0 : histData.ComanndDataList.Count));

                sb.AppendLine("");

                int num = 1;
                foreach (CommandData data in histData.ComanndDataList)
                    sb.AppendLine(string.Format("{0}.{1}", num++, data.ToString()));
            }

            this.textDetail.Text = sb.ToString();
        }

        //=====================================================================
        /// <summary>
        /// 履歴リスト選択インデックス変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listHistory_SelectedIndexChanged(object sender, EventArgs e)
        {
            // 履歴詳細設定
            this.SetHistoryDetail();
        }

        //=====================================================================
        /// <summary>
        /// 詳細テキストボックスダブルクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void textDetail_DoubleClick(object sender, EventArgs e)
        {
#if DEBUG
            // 詳細テキストの内容をクリップボードにコピーする。
            Clipboard.SetDataObject(this.textDetail.Text, false);
#endif
        }

        #endregion

        #region プロパティグリッド設定

        //=====================================================================
        /// <summary>
        /// プロパティグリッドの設定を行います。
        /// </summary>
        public void SetPropertyGrid()
        {
            if (_activePanel == null)
            {
                this.propertyItem.SelectedObject = null;
                return;
            }
            else
            {
                if (_activePanel.Drawview.Items == null || _activePanel.Drawview.SelectedItems.Count == 0)
                {
                    this.propertyItem.SelectedObject = _activePanel.Drawview.Panel;
                }
                else
                {
                    this.propertyItem.SelectedObjects = _activePanel.Drawview.SelectedItems.ToArray();
                }
            }
        }

        #endregion

        #region アイテムリスト[Del] キー押下イベント

        //=====================================================================
        /// <summary>
        /// アイテムリストの[Del] キーイベント押下イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listItem_KeyDown(object sender, KeyEventArgs e)
        {
            if (_activePanel == null)
                return;

            if (e.KeyCode == Keys.Delete)
            {
                // 選択されているアイテムを削除
                _activePanel.Drawview.RemoveSelectedItem();
            }
        }

        #endregion

        #region 印刷処理

        /// <summary>印刷列インデックス</summary>
        private int _printColIndex = 0;

        /// <summary>印刷行インデックス</summary>
        private int _printRowIndex = 0;

        //=====================================================================
        /// <summary>
        /// 印刷ドキュメントを取得します。
        /// </summary>
        /// <returns></returns>
        private PrintDocument GetPrintDocument()
        {
            PaperPanel panel = _activePanel.Drawview.Panel as PaperPanel;

            // ========== 用紙種別を求める ==========
            System.Drawing.Printing.PaperKind pk = System.Drawing.Printing.PaperKind.Custom;

            switch (panel.PaperKind)
            {
                case JsDrawControl.PaperKind.Custom:            // カスタム
                case JsDrawControl.PaperKind.PhotoE:            // E(82x120)
                case JsDrawControl.PaperKind.PhotoL:            // L(89x127)
                case JsDrawControl.PaperKind.PhotoHV:           // HV(89x152)
                case JsDrawControl.PaperKind.PhotoLP:           // LP(89x254)
                case JsDrawControl.PaperKind.PhotoKC:           // KC(100x148)
                case JsDrawControl.PaperKind.PhotoKH:           // KH(102x178)
                case JsDrawControl.PaperKind.PhotoKP:           // KP(102x305)
                case JsDrawControl.PaperKind.Photo2L:           // 2L(127x178)
                case JsDrawControl.PaperKind.Photo2LH:          // 2LH(127x226)
                case JsDrawControl.PaperKind.Photo2LP:          // 2LP(127x365)
                case JsDrawControl.PaperKind.CompactDisc:       // CD(120x120)
                case JsDrawControl.PaperKind.A0:                // A0(841x1189)
                case JsDrawControl.PaperKind.A1:                // A1(594x841)
                case JsDrawControl.PaperKind.A7:                // A7(74x105)
                case JsDrawControl.PaperKind.A8:                // A8(52x74)
                case JsDrawControl.PaperKind.A9:                // A9(37x52)
                case JsDrawControl.PaperKind.A10:               // A10(26x37)
                case JsDrawControl.PaperKind.B0:                // B0(1030x1456)
                case JsDrawControl.PaperKind.B1:                // B3(728x1030)
                case JsDrawControl.PaperKind.B2:                // B3(515x728)
                case JsDrawControl.PaperKind.B3:                // B3(364x515)
                case JsDrawControl.PaperKind.B6:                // B6(128x182)
                case JsDrawControl.PaperKind.B7:                // B7(91x128)
                case JsDrawControl.PaperKind.B8:                // B8(64x91)
                case JsDrawControl.PaperKind.B9:                // B9(45x64)
                case JsDrawControl.PaperKind.B10:               // B10(32x45)
                    pk = System.Drawing.Printing.PaperKind.Custom;
                    break;

                case JsDrawControl.PaperKind.JapanesePostCard:     // 官製はがき(107x154)
                    pk = System.Drawing.Printing.PaperKind.JapanesePostcard;
                    break;

                case JsDrawControl.PaperKind.JapanesePostCardR:    // 往復はがき(214x154)
                    pk = System.Drawing.Printing.PaperKind.JapaneseDoublePostcard;
                    break;

                case JsDrawControl.PaperKind.Letter:            // 欧米の便箋(216x280)
                    pk = System.Drawing.Printing.PaperKind.Letter;
                    break;

                case JsDrawControl.PaperKind.Legal:             // アメリカ政府の公文書(216x356)
                    pk = System.Drawing.Printing.PaperKind.Legal;
                    break;

                case JsDrawControl.PaperKind.Tabloid:           // 欧米の夕刊紙(280x432)
                    pk = System.Drawing.Printing.PaperKind.Tabloid;
                    break;

                case JsDrawControl.PaperKind.A2:                // A2(420x594)
                    pk = System.Drawing.Printing.PaperKind.A2;
                    break;

                case JsDrawControl.PaperKind.A3:                // A3(297x420)
                    pk = System.Drawing.Printing.PaperKind.A3;
                    break;

                case JsDrawControl.PaperKind.A4:                // A4(210x297)
                    pk = System.Drawing.Printing.PaperKind.A4;
                    break;

                case JsDrawControl.PaperKind.A5:                // A5(148x210)
                    pk = System.Drawing.Printing.PaperKind.A5;
                    break;

                case JsDrawControl.PaperKind.A6:                // A6(105x148)
                    pk = System.Drawing.Printing.PaperKind.A6;
                    break;

                case JsDrawControl.PaperKind.B4:                // B4(257x364)
                    pk = System.Drawing.Printing.PaperKind.B4;
                    break;

                case JsDrawControl.PaperKind.B5:                // B5(182x257)
                    pk = System.Drawing.Printing.PaperKind.B5;
                    break;
            }

            PrintDocument doc = new PrintDocument();
            doc.DefaultPageSettings.PrinterSettings.PrinterName = Program.Config.PrinterName;

            // 用紙の向きを設定(横：true、縦：false)
            doc.DefaultPageSettings.Landscape = panel.PaperOrientation == PaperOrientation.LandScape ? true : false;

            bool prtSetFlag = false;

            // ========== 用紙サイズ設定 ==========
            if (pk != System.Drawing.Printing.PaperKind.Custom)
            {
                // プリンタがサポートしている用紙サイズを調べる
                foreach (PaperSize ps in doc.PrinterSettings.PaperSizes)
                {
                    // A4用紙に設定する
                    if (ps.Kind == pk)
                    {
                        doc.DefaultPageSettings.PaperSize = ps;
                        prtSetFlag = true;
                        break;
                    }
                }
            }

            if (prtSetFlag == false)
            {
                doc.DefaultPageSettings.PaperSize =
                    new PaperSize("Custom",
                        (int)(panel.PaperSize.Width * MM_INCHE),
                        (int)(panel.PaperSize.Height * MM_INCHE));
            }

            // ========== 印刷処理 ==========
            // PrintPage イベントハンドル追加
            doc.PrintPage += new PrintPageEventHandler(doc_PrintPage);

            // ドキュメント名
            doc.DocumentName = Path.GetFileNameWithoutExtension(_activePanel.FileName);

            /// <summary>印刷インデックス初期化</summary>
            _printColIndex = 0;
            _printRowIndex = 0;

            return doc;
        }

        //=====================================================================
        /// <summary>
        /// 印刷イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void doc_PrintPage(object sender, PrintPageEventArgs e)
        {
            PaperPanel panel = _activePanel.Drawview.Panel as PaperPanel;

            // 印刷
            _activePanel.Drawview.Print(e.Graphics, panel.GetPrintBounds(_printColIndex, _printRowIndex));

            // 単位をミリメートルに設定する
            e.Graphics.PageUnit = GraphicsUnit.Millimeter;

            // 切り取り線描画
            if (Program.Config.PrintCutLine || Program.Config.PrintGlueLine)
            {
                float w = panel.PaperOrientation == PaperOrientation.Portrait ? panel.PaperSize.Width : panel.PaperSize.Height;
                float h = panel.PaperOrientation == PaperOrientation.Portrait ? panel.PaperSize.Height : panel.PaperSize.Width;
                Font drawFont = new System.Drawing.Font("MS UI Gothic", 10F);
                StringFormat sf = new StringFormat() { Alignment = StringAlignment.Center, LineAlignment = StringAlignment.Center };
                Pen drawPen = new Pen(Color.Gray, 0.1F){DashStyle = DashStyle.Dash};
                HatchBrush drawBrush = new HatchBrush(HatchStyle.BackwardDiagonal, Color.Gray, Color.Transparent);

                // 上マージン
                RectangleF marginTop = new RectangleF(
                    0.0F, 0.0F,
                    w, panel.MarginSize.Top);

                // 左マージン
                RectangleF marginLeft = new RectangleF(
                    0.0F, 0.0F,
                    panel.MarginSize.Left, h);

                // 下マージン
                RectangleF marginBottom = new RectangleF(
                    0.0F, h - panel.MarginSize.Bottom,
                    w, panel.MarginSize.Bottom);

                // 右マージン
                RectangleF marginRight = new RectangleF(
                    w - panel.MarginSize.Right, 0.0F,
                    panel.MarginSize.Right, panel.PaperSize.Height);

                // ========== 切り取り線の印刷 ==========
                if (Program.Config.PrintCutLine)
                {
                    // 下余白を切り取り
                    if (_printRowIndex < (panel.RowCount - 1))
                    {
                        e.Graphics.FillRectangle(drawBrush, marginBottom);
                        e.Graphics.DrawRectangle(drawPen, marginBottom.X, marginBottom.Y, marginBottom.Width, marginBottom.Height);
                        e.Graphics.DrawString(string.Format("きりとり"), drawFont, Brushes.Black, marginBottom, sf);
                    }

                    // 右余白を切り取り
                    if (_printColIndex < (panel.ColCount - 1))
                    {
                        e.Graphics.FillRectangle(drawBrush, marginRight);
                        e.Graphics.DrawRectangle(drawPen, marginRight.X, marginRight.Y, marginRight.Width, marginRight.Height);
                        e.Graphics.DrawString(string.Format("き\nり\nと\nり"), drawFont, Brushes.Black, marginRight, sf);
                    }
                }

                // ========== のりしろの印刷 ==========
                if (Program.Config.PrintGlueLine)
                {
                    // 上余白をのりしろ
                    if ( 0 < _printRowIndex)
                    {
                        e.Graphics.DrawRectangle(drawPen, marginTop.X, marginTop.Y, marginTop.Width, marginTop.Height);
                        e.Graphics.DrawString(string.Format("のりしろ"), drawFont, Brushes.Gray, marginTop, sf);
                    }

                    // 左余白をのりしろ
                    if (0 < _printColIndex)
                    {
                        e.Graphics.DrawRectangle(drawPen, marginLeft.X, marginLeft.Y, marginLeft.Width, marginLeft.Height);
                        e.Graphics.DrawString(string.Format("の\nり\nし\nろ"), drawFont, Brushes.Gray, marginLeft, sf);
                    }
                }
            }

            // ページインデックスを加算
            if (Program.Config.PrintOrderRightBottom)
            {
                _printColIndex++;
                if (panel.ColCount <= _printColIndex)
                {
                    _printColIndex = 0;
                    _printRowIndex++;
                }
            }
            else
            {
                _printRowIndex++;
                if (panel.RowCount <= _printRowIndex)
                {
                    _printRowIndex = 0;
                    _printColIndex++;
                }
            }

            // 印刷終わり判定
            if ((_printColIndex < panel.ColCount) && (_printRowIndex < panel.RowCount))
            {
                e.HasMorePages = true;
            }
            else
            {
                e.HasMorePages = false;
                _printColIndex = 0;
                _printRowIndex = 0;
            }
        }

        #endregion

    }
}
