﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Drawing.Printing;
using System.IO;

namespace JsDraw
{
    //=========================================================================
    /// <summary>
    /// 印刷画面フォーム
    /// </summary>
    public partial class PrintForm : Form
    {
        #region API定義

        // 指定されたプリンタまたはプリントサーバーを識別するハンドルを取得します。
        [DllImport("winspool.drv", EntryPoint = "OpenPrinterW", SetLastError = true, CharSet = CharSet.Unicode, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool OpenPrinter(string pPrinterName, out IntPtr phPrinter, IntPtr pDefault);

        // 特定のプリンタオブジェクトをクローズします。
        [DllImport("winspool.Drv", EntryPoint = "ClosePrinter", SetLastError = true, CharSet = CharSet.Unicode, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool ClosePrinter(IntPtr hPrinter);

        // 特定のプリンタのプリンタ設定プロパティシートを表示します。
        [DllImport("winspool.Drv", EntryPoint = "DocumentPropertiesW", SetLastError = true, CharSet = CharSet.Unicode, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        static extern int DocumentProperties(IntPtr hwnd, IntPtr hPrinter, [MarshalAs(UnmanagedType.LPWStr)] string pDeviceName, IntPtr pDevModeOutput, IntPtr pDevModeInput, int fMode);

        // fMode 設定値
        private const int DM_OUT_BUFFER = 0x2;
        private const int DM_PROMPT = 0x4;
        private const int DM_IN_PROMPT = DM_PROMPT;
        private const int DM_IN_BUFFER = 0x8;

        #endregion

        #region メンバ変数

        /// <summary>印刷ドキュメント</summary>
        private PrintDocument _printDoc = null;

        /// <summary>右⇒下の順で印刷を行うかどうか</summary>
        private bool _printOrderRightBottom;

        /// <summary>切り取りラインを印刷するかどうか</summary>
        private bool _printCutLine;

        /// <summary>のりしろを印刷するかどうか</summary>
        private bool _printGlueLine;

        /// <summary>プレビューを表示するかどうか</summary>
        private bool _showPreview;

        #endregion

        #region コンストラクタ、デストラクタ

        //=====================================================================
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public PrintForm()
        {
            InitializeComponent();

            // メンバ変数設定
            _printDoc = null;
            _printOrderRightBottom = true;
            _printCutLine = false;
            _printGlueLine = false;
            _showPreview = false;
        }

        #endregion

        #region パブリックプロパティ

        //=====================================================================
        /// <summary>
        /// 印刷ドキュメントプロパティ
        /// </summary>
        public PrintDocument PrintDoc
        {
            get { return _printDoc; }
            set { _printDoc = value; }
        }

        //=====================================================================
        /// <summary>
        /// 右⇒下の順で印刷を行うかどうか
        /// </summary>
        public bool PrintOrderRightBottom
        {
            get { return _printOrderRightBottom; }
            set { _printOrderRightBottom = value; }
        }

        //=====================================================================
        /// <summary>
        /// 切り取りラインを印刷するかどうか
        /// </summary>
        public bool PrintCutLine
        {
            get { return _printCutLine; }
            set { _printCutLine = value; }
        }

        //=====================================================================
        /// <summary>
        /// のりしろを印刷するかどうか
        /// </summary>
        public bool PrintGlueLine
        {
            get { return _printGlueLine; }
            set { _printGlueLine = value; }
        }

        //=====================================================================
        /// <summary>
        /// プレビューを表示するかどうか
        /// </summary>
        public bool ShowPreview
        {
            get { return _showPreview; }
            set { _showPreview = value; }
        }

        #endregion

        #region ロードイベント処理

        //=====================================================================
        /// <summary>
        /// フォームロードイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PrintForm_Load(object sender, EventArgs e)
        {
            // ========== 画面設定 ==========

            // プリンタ一覧
            this.comboPrinter.Items.Clear();
            int index = 0;
            for (int i = 0; i < PrinterSettings.InstalledPrinters.Count; i++)
            {
                // プリンタ名
                string printerName = PrinterSettings.InstalledPrinters[i].ToString();
                this.comboPrinter.Items.Add(printerName);

                if (printerName.Equals(_printDoc.PrinterSettings.PrinterName))
                    index = i;                
            }
            this.comboPrinter.SelectedIndex = index;

            // 印刷方向
            if (_printOrderRightBottom)
            {
                this.radioOrderRB.Checked = true;
                this.radioOrderBR.Checked = false;
            }
            else
            {
                this.radioOrderRB.Checked = false;
                this.radioOrderBR.Checked = true;
            }

            // 余白の印刷
            this.checkPrintCutLine.Checked = _printCutLine;
            this.checkPrintGlueLine.Checked = _printGlueLine;
        }

        #endregion

        #region プリンタの設定

        //=====================================================================
        /// <summary>
        /// 「プリンタの設定」ボタンクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPrinterSetting_Click(object sender, EventArgs e)
        {
            IntPtr devModeIn = IntPtr.Zero;
            IntPtr devModeOut = IntPtr.Zero;
            IntPtr hPrinter = IntPtr.Zero;

            try
            {
                string printerName = _printDoc.PrinterSettings.PrinterName;

                // グローバルメモリオブジェクトをロックします。
                devModeIn = LoadPrinterSetting(printerName);
                if (devModeIn == IntPtr.Zero)
                {
                    devModeIn = Marshal.AllocHGlobal(_printDoc.PrinterSettings.GetHdevmode());
                }
                devModeOut = Marshal.AllocHGlobal(_printDoc.PrinterSettings.GetHdevmode());

                //プリンタのハンドルを取得
                if (!OpenPrinter(printerName, out hPrinter, IntPtr.Zero))
                {
                    return;
                }

                // DEVMODE サイズ取得
                int devSize = DocumentProperties(this.Handle, hPrinter, printerName,
                    IntPtr.Zero, IntPtr.Zero, 0);

                // プリンタの設定画面表示
                if (DocumentProperties(this.Handle, hPrinter, printerName,
                    devModeOut, devModeIn, DM_IN_PROMPT | DM_IN_BUFFER | DM_OUT_BUFFER) == 1)
                {

                    // ＯＫボタンがクリックされた場合
                    _printDoc.PrinterSettings.SetHdevmode(devModeOut);

                    // DEVMODE をコピー
                    byte[] setting = new byte[devSize];
                    Marshal.Copy(devModeOut, setting, 0, devSize);

                    // 設定ファイル名
                    string fileName = Path.Combine(Application.StartupPath, printerName + ".dat");

                    // ファイルオープン
                    using (FileStream fs = new FileStream(fileName, FileMode.Create, FileAccess.Write))
                    {
                        // 書き込み
                        fs.Write(setting, 0, setting.Length);

                        // ファイルクローズ
                        fs.Close();
                    }
                }
                else
                {
                    return;
                }
            }
            finally
            {
                // グロバールメモリ開放
                Marshal.FreeHGlobal(devModeIn);
                Marshal.FreeHGlobal(devModeOut);
                ClosePrinter(hPrinter);
            }
        }

        #endregion

        #region プリンタコンボボックス変更イベント処理

        //=====================================================================
        /// <summary>
        /// プリンタコンボボックス変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboPrinter_SelectedIndexChanged(object sender, EventArgs e)
        {
            // プリンタ名設定
            string printerName = this.comboPrinter.Text;
            _printDoc.PrinterSettings.PrinterName = printerName;

            // プリンタ情報設定
            IntPtr devMode = LoadPrinterSetting(printerName);
            if (devMode != IntPtr.Zero)
            {
                _printDoc.PrinterSettings.SetHdevmode(devMode);
                Marshal.FreeHGlobal(devMode);
            }
        }

        #endregion

        #region プリンタ設定のロードと保存

        //=====================================================================
        /// <summary>
        /// プリンタ設定のロード
        /// </summary>
        /// <param name="printerName"></param>
        /// <returns></returns>
        IntPtr LoadPrinterSetting(string printerName)
        {
            IntPtr devPtr = IntPtr.Zero;

            // 設定ファイル名
            string fileName = Path.Combine(Application.StartupPath, printerName + ".dat");

            // ファイルが存在する場合
            if (File.Exists(fileName))
            {
                using (FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read))
                {
                    // 読み込み
                    byte[] setting = new byte[fs.Length];
                    fs.Read(setting, 0, (int)fs.Length);

                    // グローバル領域にメモリー確保
                    devPtr = Marshal.AllocHGlobal(setting.Length);

                    // 設定コピー
                    Marshal.Copy(setting, 0, devPtr, setting.Length);

                    // ファイルクローズ
                    fs.Close();
                }
            }

            return devPtr;
        }

        //=====================================================================
        /// <summary>
        /// プリンタ設定のロード
        /// </summary>
        /// <param name="doc"></param>
        /// <returns></returns>
        IntPtr LoadPrinterSetting(PrintDocument doc)
        {
            IntPtr devPtr;

            // 設定ファイル名
            string fileName = Path.Combine(Application.StartupPath,
                doc.PrinterSettings.PrinterName + ".dat");

            // ファイルが存在する場合
            if (File.Exists(fileName))
            {
                using (FileStream fs = new FileStream(
                    fileName, FileMode.Open, FileAccess.Read))
                {
                    // 読み込み
                    byte[] setting = new byte[fs.Length];
                    fs.Read(setting, 0, (int)fs.Length);

                    // グローバル領域にメモリー確保
                    devPtr = Marshal.AllocHGlobal(setting.Length);

                    // 設定コピー
                    Marshal.Copy(setting, 0, devPtr, setting.Length);

                    // ファイルクローズ
                    fs.Close();
                }
            }
            else
            {
                devPtr = Marshal.AllocHGlobal(doc.PrinterSettings.GetHdevmode());
            }

            return devPtr;
        }

        //=====================================================================
        /// <summary>
        /// プリンタ設定の保存
        /// </summary>
        /// <param name="hPrinter"></param>
        /// <param name="devMode"></param>
        /// <param name="doc"></param>
        void SavePrinterSetting(IntPtr hPrinter, IntPtr devMode, PrintDocument doc)
        {
            // DEVMODE サイズ取得
            int devSize = DocumentProperties(
                this.Handle, hPrinter, doc.PrinterSettings.PrinterName,
                IntPtr.Zero, IntPtr.Zero, 0);

            // DEVMODE をコピー
            byte[] setting = new byte[devSize];
            Marshal.Copy(devMode, setting, 0, devSize);

            // 設定ファイル名
            string fileName = Path.Combine(Application.StartupPath,
                doc.PrinterSettings.PrinterName + ".dat");

            // ファイルオープン
            using (FileStream fs = new FileStream(
                fileName, FileMode.Create, FileAccess.Write))
            {
                // 書き込み
                fs.Write(setting, 0, setting.Length);

                // ファイルクローズ
                fs.Close();
            }
        }

        #endregion

        #region 「印刷」ボタンクリックイベント処理

        //=====================================================================
        /// <summary>
        /// 「印刷」ボタンクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPrinting_Click(object sender, EventArgs e)
        {
            // 値設定
            _printOrderRightBottom = this.radioOrderRB.Checked;
            _printCutLine = this.checkPrintCutLine.Checked;
            _printGlueLine = this.checkPrintGlueLine.Checked;
            _showPreview = false;

            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        #endregion

        #region 「プレビュー」ボタンクリックイベント処理

        //=====================================================================
        /// <summary>
        /// 「プレビュー」ボタンクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPreview_Click(object sender, EventArgs e)
        {
            // 値設定
            _printOrderRightBottom = this.radioOrderRB.Checked;
            _printCutLine = this.checkPrintCutLine.Checked;
            _printGlueLine = this.checkPrintGlueLine.Checked;
            _showPreview = true;

            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        #endregion

        #region 「閉じる」ボタンクリックイベント処理

        //=====================================================================
        /// <summary>
        /// 「閉じる」ボタンクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Abort;
            this.Close();
        }

        #endregion
    }
}
